@@ -1,39 +1,38 @@
 """Certbot command line argument parser"""
 
 import argparse
-import copy
 import functools
 import glob
 import sys
 from typing import Any
 from typing import Dict
+from typing import Iterable
+from typing import List
+from typing import Optional
+from typing import Tuple
+from typing import Union
 
 import configargparse
-import zope.component
-import zope.interface
-from zope.interface import interfaces as zope_interfaces
 
 from certbot import crypto_util
 from certbot import errors
-from certbot import interfaces
 from certbot import util
 from certbot._internal import constants
 from certbot._internal import hooks
-from certbot._internal.cli.cli_constants import ARGPARSE_PARAMS_TO_REMOVE
 from certbot._internal.cli.cli_constants import COMMAND_OVERVIEW
-from certbot._internal.cli.cli_constants import EXIT_ACTIONS
 from certbot._internal.cli.cli_constants import HELP_AND_VERSION_USAGE
 from certbot._internal.cli.cli_constants import SHORT_USAGE
-from certbot._internal.cli.cli_constants import ZERO_ARG_ACTIONS
-from certbot._internal.cli.cli_utils import _Default
 from certbot._internal.cli.cli_utils import add_domains
 from certbot._internal.cli.cli_utils import CustomHelpFormatter
 from certbot._internal.cli.cli_utils import flag_default
 from certbot._internal.cli.cli_utils import HelpfulArgumentGroup
 from certbot._internal.cli.verb_help import VERB_HELP
 from certbot._internal.cli.verb_help import VERB_HELP_MAP
+from certbot._internal.display import obj as display_obj
+from certbot._internal.plugins import disco
 from certbot.compat import os
-from certbot.display import util as display_util
+from certbot.configuration import ArgumentSource
+from certbot.configuration import NamespaceConfig
 
 
 class HelpfulArgumentParser:
@@ -44,7 +43,7 @@ class HelpfulArgumentParser:
     'certbot --help security' for security options.
 
     """
-    def __init__(self, args, plugins, detect_defaults=False):
+    def __init__(self, args: List[str], plugins: Iterable[str]) -> None:
         from certbot._internal import main
         self.VERBS = {
             "auth": main.certonly,
@@ -54,6 +53,7 @@ class HelpfulArgumentParser:
             "plugins": main.plugins_cmd,
             "register": main.register,
             "update_account": main.update_account,
+            "show_account": main.show_account,
             "unregister": main.unregister,
             "renew": main.renew,
             "revoke": main.revoke,
@@ -63,25 +63,21 @@ class HelpfulArgumentParser:
             "certificates": main.certificates,
             "delete": main.delete,
             "enhance": main.enhance,
+            "reconfigure": main.reconfigure,
         }
 
         # Get notification function for printing
-        try:
-            self.notify = zope.component.getUtility(
-                interfaces.IDisplay).notification
-        except zope_interfaces.ComponentLookupError:
-            self.notify = display_util.NoninteractiveDisplay(
-                sys.stdout).notification
+        self.notify = display_obj.NoninteractiveDisplay(sys.stdout).notification
 
+        self.actions: List[configargparse.Action] = []
 
         # List of topics for which additional help can be provided
-        HELP_TOPICS = ["all", "security", "paths", "automation", "testing"]
+        HELP_TOPICS: List[Optional[str]] = ["all", "security", "paths", "automation", "testing"]
         HELP_TOPICS += list(self.VERBS) + self.COMMANDS_TOPICS + ["manage"]
 
-        plugin_names = list(plugins)
-        self.help_topics = HELP_TOPICS + plugin_names + [None]  # type: ignore
+        plugin_names: List[Optional[str]] = list(plugins)
+        self.help_topics: List[Optional[str]] = HELP_TOPICS + plugin_names + [None]
 
-        self.detect_defaults = detect_defaults
         self.args = args
 
         if self.args and self.args[0] == 'help':
@@ -90,6 +86,7 @@ class HelpfulArgumentParser:
         self.determine_verb()
         help1 = self.prescan_for_flag("-h", self.help_topics)
         help2 = self.prescan_for_flag("--help", self.help_topics)
+        self.help_arg: Union[str, bool]
         if isinstance(help1, bool) and isinstance(help2, bool):
             self.help_arg = help1 or help2
         else:
@@ -101,8 +98,6 @@ class HelpfulArgumentParser:
 
         # elements are added by .add_group()
         self.groups: Dict[str, argparse._ArgumentGroup] = {}
-        # elements are added by .parse_args()
-        self.defaults: Dict[str, Any] = {}
 
         self.parser = configargparse.ArgParser(
             prog="certbot",
@@ -121,7 +116,7 @@ class HelpfulArgumentParser:
     # Help that are synonyms for --help subcommands
     COMMANDS_TOPICS = ["command", "commands", "subcommand", "subcommands", "verbs"]
 
-    def _list_subcommands(self):
+    def _list_subcommands(self) -> str:
         longest = max(len(v) for v in VERB_HELP_MAP)
 
         text = "The full list of available SUBCOMMANDS is:\n\n"
@@ -132,7 +127,7 @@ class HelpfulArgumentParser:
         text += "\nYou can get more help on a specific subcommand with --help SUBCOMMAND\n"
         return text
 
-    def _usage_string(self, plugins, help_arg):
+    def _usage_string(self, plugins: Iterable[str], help_arg: Union[str, bool]) -> str:
         """Make usage strings late so that plugins can be initialised late
 
         :param plugins: all discovered plugins
@@ -160,81 +155,166 @@ class HelpfulArgumentParser:
             # if we're doing --help all, the OVERVIEW is part of the SHORT_USAGE at
             # the top; if we're doing --help someothertopic, it's OT so it's not
             usage += COMMAND_OVERVIEW % (apache_doc, nginx_doc)
-        else:
+        elif isinstance(help_arg, str):
             custom = VERB_HELP_MAP.get(help_arg, {}).get("usage", None)
             usage = custom if custom else usage
+        # Only remaining case is help_arg == False, which gives effectively usage == SHORT_USAGE.
 
         return usage
 
-    def remove_config_file_domains_for_renewal(self, parsed_args):
+    def remove_config_file_domains_for_renewal(self, config: NamespaceConfig) -> None:
         """Make "certbot renew" safe if domains are set in cli.ini."""
         # Works around https://github.com/certbot/certbot/issues/4096
-        if self.verb == "renew":
-            for source, flags in self.parser._source_to_settings.items(): # pylint: disable=protected-access
-                if source.startswith("config_file") and "domains" in flags:
-                    parsed_args.domains = _Default() if self.detect_defaults else []
+        if (config.argument_sources['domains'] == ArgumentSource.CONFIG_FILE and
+                self.verb == "renew"):
+            config.domains = []
 
-    def parse_args(self):
+    def _build_sources_dict(self) -> Dict[str, ArgumentSource]:
+        # ConfigArgparse's get_source_to_settings_dict doesn't actually create
+        # default entries for each argument with a default value, omitting many
+        # args we'd otherwise care about. So in general, unless an argument was
+        # specified in a config file/environment variable/command line arg,
+        # consider it as having a "default" value
+        result = { action.dest: ArgumentSource.DEFAULT for action in self.actions }
+
+        source_to_settings_dict: Dict[str, Dict[str, Tuple[configargparse.Action, str]]]
+        source_to_settings_dict = self.parser.get_source_to_settings_dict()
+
+        # We'll process the sources dict in order of each source's "priority",
+        # i.e. the order in which ConfigArgparse ultimately sets argument
+        # values:
+        #   1. defaults (`result` already has everything marked as such)
+        #   2. config files
+        #   3. env vars (shouldn't be any)
+        #   4. command line
+
+        def update_result(settings_dict: Dict[str, Tuple[configargparse.Action, str]],
+                          source: ArgumentSource) -> None:
+            actions = [self._find_action_for_arg(arg) if action is None else action
+                       for arg, (action, _) in settings_dict.items()]
+            result.update({ action.dest: source for action in actions })
+
+        # config file sources look like "config_file|<name of file>"
+        for source_key in source_to_settings_dict:
+            if source_key.startswith('config_file'):
+                update_result(source_to_settings_dict[source_key], ArgumentSource.CONFIG_FILE)
+
+        update_result(source_to_settings_dict.get('env_var', {}), ArgumentSource.ENV_VAR)
+
+        # The command line settings dict is weird, so handle it separately
+        if 'command_line' in source_to_settings_dict:
+            settings_dict: Dict[str, Tuple[None, List[str]]]
+            settings_dict = source_to_settings_dict['command_line'] # type: ignore
+            (_, unprocessed_args) = settings_dict['']
+            args = []
+            for arg in unprocessed_args:
+                # ignore non-arguments
+                if not arg.startswith('-'):
+                    continue
+
+                # special case for config file argument, which we don't have an action for
+                if arg in ['-c', '--config']:
+                    result['config_dir'] = ArgumentSource.COMMAND_LINE
+                    continue
+
+                if '=' in arg:
+                    arg = arg.split('=')[0]
+
+                if arg.startswith('--'):
+                    args.append(arg)
+                # for short args (ones that start with a single hyphen), handle
+                # the case of multiple short args together, e.g. "-tvm"
+                else:
+                    for short_arg in arg[1:]:
+                        args.append(f"-{short_arg}")
+
+            for arg in args:
+                # find the action corresponding to this arg
+                action = self._find_action_for_arg(arg)
+                result[action.dest] = ArgumentSource.COMMAND_LINE
+
+        return result
+
+    def _find_action_for_arg(self, arg: str) -> configargparse.Action:
+        # Finds a configargparse Action which matches the given arg, where arg
+        # can either be preceded by hyphens (as on the command line) or not (as
+        # in config files)
+
+        # if the argument doesn't have leading hypens, prefix it so it can be
+        # compared directly w/ action option strings
+        if arg[0] != '-':
+            arg = '--' + arg
+
+        # first, check for exact matches
+        for action in self.actions:
+            if arg in action.option_strings:
+                return action
+
+        # now check for abbreviated (i.e. prefix) matches
+        for action in self.actions:
+            for option_string in action.option_strings:
+                if option_string.startswith(arg):
+                    return action
+
+        raise AssertionError(f"Action corresponding to argument {arg} is None")
+
+    def parse_args(self) -> NamespaceConfig:
         """Parses command line arguments and returns the result.
 
         :returns: parsed command line arguments
-        :rtype: argparse.Namespace
+        :rtype: configuration.NamespaceConfig
 
         """
         parsed_args = self.parser.parse_args(self.args)
         parsed_args.func = self.VERBS[self.verb]
         parsed_args.verb = self.verb
+        config = NamespaceConfig(parsed_args)
+        config.set_argument_sources(self._build_sources_dict())
 
-        self.remove_config_file_domains_for_renewal(parsed_args)
-
-        if self.detect_defaults:
-            return parsed_args
-
-        self.defaults = {key: copy.deepcopy(self.parser.get_default(key))
-                             for key in vars(parsed_args)}
+        self.remove_config_file_domains_for_renewal(config)
 
         # Do any post-parsing homework here
 
         if self.verb == "renew":
-            if parsed_args.force_interactive:
+            if config.force_interactive:
                 raise errors.Error(
                     "{0} cannot be used with renew".format(
                         constants.FORCE_INTERACTIVE_FLAG))
-            parsed_args.noninteractive_mode = True
+            config.noninteractive_mode = True
 
-        if parsed_args.force_interactive and parsed_args.noninteractive_mode:
+        if config.force_interactive and config.noninteractive_mode:
             raise errors.Error(
                 "Flag for non-interactive mode and {0} conflict".format(
                     constants.FORCE_INTERACTIVE_FLAG))
 
-        if parsed_args.staging or parsed_args.dry_run:
-            self.set_test_server(parsed_args)
+        if config.staging or config.dry_run:
+            self.set_test_server(config)
 
-        if parsed_args.csr:
-            self.handle_csr(parsed_args)
+        if config.csr:
+            self.handle_csr(config)
 
-        if parsed_args.must_staple:
-            parsed_args.staple = True
+        if config.must_staple and not config.staple:
+            config.staple = True
 
-        if parsed_args.validate_hooks:
-            hooks.validate_hooks(parsed_args)
+        if config.validate_hooks:
+            hooks.validate_hooks(config)
 
-        if parsed_args.allow_subset_of_names:
-            if any(util.is_wildcard_domain(d) for d in parsed_args.domains):
+        if config.allow_subset_of_names:
+            if any(util.is_wildcard_domain(d) for d in config.domains):
                 raise errors.Error("Using --allow-subset-of-names with a"
                                    " wildcard domain is not supported.")
 
-        if parsed_args.hsts and parsed_args.auto_hsts:
+        if config.hsts and config.auto_hsts:
             raise errors.Error(
                 "Parameters --hsts and --auto-hsts cannot be used simultaneously.")
 
-        if isinstance(parsed_args.key_type, list) and len(parsed_args.key_type) > 1:
+        if isinstance(config.key_type, list) and len(config.key_type) > 1:
             raise errors.Error(
                 "Only *one* --key-type type may be provided at this time.")
 
-        return parsed_args
+        return config
 
-    def set_test_server(self, parsed_args):
+    def set_test_server(self, config: NamespaceConfig) -> None:
         """We have --staging/--dry-run; perform sanity check and set config.server"""
 
         # Flag combinations should produce these results:
@@ -246,58 +326,58 @@ class HelpfulArgumentParser:
 
         default_servers = (flag_default("server"), constants.STAGING_URI)
 
-        if parsed_args.staging and parsed_args.server not in default_servers:
+        if config.staging and config.server not in default_servers:
             raise errors.Error("--server value conflicts with --staging")
 
-        if parsed_args.server in default_servers:
-            parsed_args.server = constants.STAGING_URI
+        if config.server == flag_default("server"):
+            config.server = constants.STAGING_URI
 
-        if parsed_args.dry_run:
+        if config.dry_run:
             if self.verb not in ["certonly", "renew"]:
                 raise errors.Error("--dry-run currently only works with the "
                                    "'certonly' or 'renew' subcommands (%r)" % self.verb)
-            parsed_args.break_my_certs = parsed_args.staging = True
-            if glob.glob(os.path.join(parsed_args.config_dir, constants.ACCOUNTS_DIR, "*")):
+            config.break_my_certs = config.staging = True
+            if glob.glob(os.path.join(config.config_dir, constants.ACCOUNTS_DIR, "*")):
                 # The user has a prod account, but might not have a staging
                 # one; we don't want to start trying to perform interactive registration
-                parsed_args.tos = True
-                parsed_args.register_unsafely_without_email = True
+                config.tos = True
+                config.register_unsafely_without_email = True
 
-    def handle_csr(self, parsed_args):
+    def handle_csr(self, config: NamespaceConfig) -> None:
         """Process a --csr flag."""
-        if parsed_args.verb != "certonly":
+        if config.verb != "certonly":
             raise errors.Error("Currently, a CSR file may only be specified "
                                "when obtaining a new or replacement "
                                "via the certonly command. Please try the "
                                "certonly command instead.")
-        if parsed_args.allow_subset_of_names:
+        if config.allow_subset_of_names:
             raise errors.Error("--allow-subset-of-names cannot be used with --csr")
 
-        csrfile, contents = parsed_args.csr[0:2]
+        csrfile, contents = config.csr[0:2]
         typ, csr, domains = crypto_util.import_csr_file(csrfile, contents)
 
         # This is not necessary for webroot to work, however,
-        # obtain_certificate_from_csr requires parsed_args.domains to be set
+        # obtain_certificate_from_csr requires config.domains to be set
         for domain in domains:
-            add_domains(parsed_args, domain)
+            add_domains(config, domain)
 
         if not domains:
             # TODO: add CN to domains instead:
             raise errors.Error(
                 "Unfortunately, your CSR %s needs to have a SubjectAltName for every domain"
-                % parsed_args.csr[0])
+                % config.csr[0])
 
-        parsed_args.actual_csr = (csr, typ)
+        config.actual_csr = (csr, typ)
 
         csr_domains = {d.lower() for d in domains}
-        config_domains = set(parsed_args.domains)
+        config_domains = set(config.domains)
         if csr_domains != config_domains:
             raise errors.ConfigurationError(
                 "Inconsistent domain requests:\nFrom the CSR: {0}\nFrom command line/config: {1}"
                 .format(", ".join(csr_domains), ", ".join(config_domains)))
 
 
-    def determine_verb(self):
+    def determine_verb(self) -> None:
         """Determines the verb/subcommand provided by the user.
 
         This function works around some of the limitations of argparse.
@@ -321,7 +401,8 @@ class HelpfulArgumentParser:
 
         self.verb = "run"
 
-    def prescan_for_flag(self, flag, possible_arguments):
+    def prescan_for_flag(self, flag: str, possible_arguments: Iterable[Optional[str]]
+                         ) -> Union[str, bool]:
         """Checks cli input for flags.
 
         Check for a flag, which accepts a fixed set of possible arguments, in
@@ -342,7 +423,8 @@ class HelpfulArgumentParser:
             pass
         return True
 
-    def add(self, topics, *args, **kwargs):
+    def add(self, topics: Optional[Union[List[Optional[str]], str]], *args: Any,
+            **kwargs: Any) -> None:
         """Add a new command line argument.
 
         :param topics: str or [str] help topic(s) this should be listed under,
@@ -354,6 +436,10 @@ class HelpfulArgumentParser:
         :param dict **kwargs: various argparse settings for this argument
 
         """
+        self.actions.append(self._add(topics, *args, **kwargs))
+
+    def _add(self, topics: Optional[Union[List[Optional[str]], str]], *args: Any,
+            **kwargs: Any) -> configargparse.Action:
         action = kwargs.get("action")
         if action is util.DeprecatedArgumentAction:
             # If the argument is deprecated through
@@ -364,8 +450,7 @@ class HelpfulArgumentParser:
             # handling default detection since these actions aren't needed and
             # can cause bugs like
             # https://github.com/certbot/certbot/issues/8495.
-            self.parser.add_argument(*args, **kwargs)
-            return
+            return self.parser.add_argument(*args, **kwargs)
 
         if isinstance(topics, list):
             # if this flag can be listed in multiple sections, try to pick the one
@@ -374,42 +459,17 @@ class HelpfulArgumentParser:
         else:
             topic = topics  # there's only one
 
-        if self.detect_defaults:
-            kwargs = self.modify_kwargs_for_default_detection(**kwargs)
-
-        if self.visible_topics[topic]:
+        if not isinstance(topic, bool) and self.visible_topics[topic]:
             if topic in self.groups:
                 group = self.groups[topic]
-                group.add_argument(*args, **kwargs)
+                return group.add_argument(*args, **kwargs)
             else:
-                self.parser.add_argument(*args, **kwargs)
+                return self.parser.add_argument(*args, **kwargs)
         else:
             kwargs["help"] = argparse.SUPPRESS
-            self.parser.add_argument(*args, **kwargs)
+            return self.parser.add_argument(*args, **kwargs)
 
-    def modify_kwargs_for_default_detection(self, **kwargs):
-        """Modify an arg so we can check if it was set by the user.
-
-        Changes the parameters given to argparse when adding an argument
-        so we can properly detect if the value was set by the user.
-
-        :param dict kwargs: various argparse settings for this argument
-
-        :returns: a modified versions of kwargs
-        :rtype: dict
-
-        """
-        action = kwargs.get("action", None)
-        if action not in EXIT_ACTIONS:
-            kwargs["action"] = ("store_true" if action in ZERO_ARG_ACTIONS else
-                                "store")
-            kwargs["default"] = _Default()
-            for param in ARGPARSE_PARAMS_TO_REMOVE:
-                kwargs.pop(param, None)
-
-        return kwargs
-
-    def add_deprecated_argument(self, argument_name, num_args):
+    def add_deprecated_argument(self, argument_name: str, num_args: int) -> None:
         """Adds a deprecated argument with the name argument_name.
 
         Deprecated arguments are not shown in the help. If they are used
@@ -417,7 +477,7 @@ class HelpfulArgumentParser:
         argument is deprecated and no other action is taken.
 
         :param str argument_name: Name of deprecated argument.
-        :param int nargs: Number of arguments the option takes.
+        :param int num_args: Number of arguments the option takes.
 
         """
         # certbot.util.add_deprecated_argument expects the normal add_argument
@@ -437,7 +497,8 @@ class HelpfulArgumentParser:
         add_func = functools.partial(self.add, None)
         util.add_deprecated_argument(add_func, argument_name, num_args)
 
-    def add_group(self, topic, verbs=(), **kwargs):
+    def add_group(self, topic: str, verbs: Iterable[str] = (),
+                  **kwargs: Any) -> HelpfulArgumentGroup:
         """Create a new argument group.
 
         This method must be called once for every topic, however, calls
@@ -459,7 +520,7 @@ class HelpfulArgumentParser:
                     self.groups[topic].add_argument(v, help=VERB_HELP_MAP[v]["short"])
         return HelpfulArgumentGroup(self, topic)
 
-    def add_plugin_args(self, plugins):
+    def add_plugin_args(self, plugins: disco.PluginsRegistry) -> None:
         """
 
         Let each of the plugins add its own command line arguments, which
@@ -471,7 +532,8 @@ class HelpfulArgumentParser:
                                              description=plugin_ep.long_description)
             plugin_ep.plugin_cls.inject_parser_options(parser_or_group, name)
 
-    def determine_help_topics(self, chosen_topic):
+    def determine_help_topics(self, chosen_topic: Union[str, bool]
+                              ) -> Dict[Optional[str], bool]:
         """
 
         The user may have requested help on a topic, return a dict of which
