@@ -1,21 +1,27 @@
 """Certbot constants."""
+import atexit
 import logging
+import sys
+from contextlib import ExitStack
 from typing import Any
 from typing import Dict
 
-import pkg_resources
-
 from acme import challenges
 from certbot.compat import misc
 from certbot.compat import os
 
+if sys.version_info >= (3, 9):  # pragma: no cover
+    import importlib.resources as importlib_resources
+else:  # pragma: no cover
+    import importlib_resources
+
 SETUPTOOLS_PLUGINS_ENTRY_POINT = "certbot.plugins"
 """Setuptools entry point group name for plugins."""
 
 OLD_SETUPTOOLS_PLUGINS_ENTRY_POINT = "letsencrypt.plugins"
 """Plugins Setuptools entry point before rename."""
 
-CLI_DEFAULTS: Dict[str, Any] = dict(
+CLI_DEFAULTS: Dict[str, Any] = dict(  # noqa
     config_files=[
         os.path.join(misc.get_default_folder('config'), 'cli.ini'),
         # https://freedesktop.org/wiki/Software/xdg-user-dirs/
@@ -61,7 +67,7 @@ CLI_DEFAULTS: Dict[str, Any] = dict(
     break_my_certs=False,
     rsa_key_size=2048,
     elliptic_curve="secp256r1",
-    key_type="rsa",
+    key_type="ecdsa",
     must_staple=False,
     redirect=None,
     auto_hsts=False,
@@ -74,11 +80,13 @@ CLI_DEFAULTS: Dict[str, Any] = dict(
     validate_hooks=True,
     directory_hooks=True,
     reuse_key=False,
+    new_key=False,
     disable_renew_updates=False,
     random_sleep_on_renew=True,
     eab_hmac_key=None,
     eab_kid=None,
     issuance_timeout=90,
+    run_deploy_hooks=False,
 
     # Subparsers
     num=None,
@@ -111,7 +119,6 @@ CLI_DEFAULTS: Dict[str, Any] = dict(
     manual=False,
     webroot=False,
     dns_cloudflare=False,
-    dns_cloudxns=False,
     dns_digitalocean=False,
     dns_dnsimple=False,
     dns_dnsmadeeasy=False,
@@ -149,14 +156,10 @@ QUIET_LOGGING_LEVEL = logging.ERROR
 DEFAULT_LOGGING_LEVEL = logging.WARNING
 """Default logging level to use when not in quiet mode."""
 
-RENEWER_DEFAULTS = dict(
-    renewer_enabled="yes",
-    renew_before_expiry="30 days",
-    # This value should ensure that there is never a deployment delay by
-    # default.
-    deploy_before_expiry="99 years",
-)
-"""Defaults for renewer script."""
+RENEWER_DEFAULTS = {
+    "renew_before_expiry": "30 days",
+}
+"""Defaults for `certbot renew`."""
 
 ARCHIVE_DIR = "archive"
 """Archive directory, relative to `certbot.configuration.NamespaceConfig.config_dir`."""
@@ -223,8 +226,15 @@ SSL_DHPARAMS_DEST = "ssl-dhparams.pem"
 """Name of the ssl_dhparams file as saved
 in `certbot.configuration.NamespaceConfig.config_dir`."""
 
-SSL_DHPARAMS_SRC = pkg_resources.resource_filename(
-    "certbot", "ssl-dhparams.pem")
+def _generate_ssl_dhparams_src_static() -> str:
+    # This code ensures that the resource is accessible as file for the lifetime of current
+    # Python process, and will be automatically cleaned up on exit.
+    file_manager = ExitStack()
+    atexit.register(file_manager.close)
+    ssl_dhparams_src_ref = importlib_resources.files("certbot") / "ssl-dhparams.pem"
+    return str(file_manager.enter_context(importlib_resources.as_file(ssl_dhparams_src_ref)))
+
+SSL_DHPARAMS_SRC = _generate_ssl_dhparams_src_static()
 """Path to the nginx ssl_dhparams file found in the Certbot distribution."""
 
 UPDATED_SSL_DHPARAMS_DIGEST = ".updated-ssl-dhparams-pem-digest.txt"
