@@ -2,6 +2,7 @@
 from typing import Any
 from typing import Mapping
 from typing import TYPE_CHECKING
+from unittest import mock
 
 import configobj
 import josepy as jose
@@ -19,12 +20,6 @@ else:
     Protocol = object
 
 
-try:
-    import mock
-except ImportError:  # pragma: no cover
-    from unittest import mock  # type: ignore
-
-
 DOMAIN = 'example.com'
 KEY = jose.JWKRSA.load(test_util.load_vector("rsa512_key.pem"))
 
@@ -45,6 +40,12 @@ class _AuthenticatorCallableTestCase(Protocol):
         https://docs.python.org/3/library/unittest.html#unittest.TestCase.assertEqual
         """
 
+    def assertRaises(self, *unused_args: Any) -> None:
+        """
+        See
+        https://docs.python.org/3/library/unittest.html#unittest.TestCase.assertRaises
+        """
+
 
 class BaseAuthenticatorTest:
     """
